/*
 * Copyright (c) 2018 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.kanime.activities

import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.Menu
import android.view.MenuItem
import android.widget.ImageView
import android.widget.TextView
import com.raywenderlich.kanime.R
import org.jetbrains.anko.*


class AnimeDetailActivity : AppCompatActivity() {

  private lateinit var title: String
  private lateinit var description: String
  private lateinit var imdbLink: String
  private var imageResDrawable: Int = 0

  private lateinit var imageView: ImageView
  private lateinit var titleTextView: TextView
  private lateinit var descriptionTextView: TextView
  private lateinit var loveItImageView: ImageView
  private lateinit var hateItImageView: ImageView

  companion object {
    //Love it constants
    private val INDEX_WATCH_IT_IN_DAY = 0
    private val INDEX_THE_BEST_OF_THE_BEST = 1
    private val INDEX_SEND_MAIL = 2

    //Hate it constants
    private val INDEX_BORED_TO_DEATH = 0
    private val INDEX_MY_GRANDMA = 1

  }

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    setContentView(R.layout.activity_anime_detail)

    initViewElements()

    extractAnimeFrom(intent)

    bindingAnimeUI()

    settingListeners()
  }

  private fun settingListeners() {
    loveItImageView.setOnClickListener {

      val options = listOf(getString(R.string.watched_id_in_a_day), getString(R.string.the_best), getString(R.string.tell_me_more_by_email))

      showSelector(getString(R.string.tell_me_more), options) { dialogInterface, selectedIndex ->
        when (selectedIndex) {
          INDEX_WATCH_IT_IN_DAY -> {
            //Easy and readable way to present a toast
            //Also, you can pass the String value instead of the string id resource
            showShortToast(getString(R.string.me_too))
          }
          INDEX_THE_BEST_OF_THE_BEST -> {
            // Did you remember Toast.makeText(context, message, Toast.LENGTH_LONG).show()?
            //  I don't
            showLongToast(getString(R.string.completely_agree))
          }
          INDEX_SEND_MAIL -> sendAnimeByEmail(getString(R.string.my_email), getString(R.string.template_email_body, title), getString(R.string.template_email_body, title))
        }
      }
    }

    hateItImageView.setOnClickListener {

      val options = listOf(getString(R.string.it_bored_to_death), getString(R.string.my_grandma_thinks_is_cool), getString(R.string.tell_me_more_by_email))

      //Creates a dialog with multiple options
      showSelector(getString(R.string.tell_me_more), options) { dialogInterface, selectedIndex ->

        when (selectedIndex) {
          INDEX_BORED_TO_DEATH -> showShortToast(getString(R.string.im_not_agree))
          INDEX_SEND_MAIL -> sendAnimeByEmail(getString(R.string.my_email), getString(R.string.template_negative_email_body, title), getString(R.string.template_negative_email_body, title))
          INDEX_MY_GRANDMA -> {

            showAlert(R.string.seriously, onYesTapped = {
              showShortToast(getString(R.string.smiley_face))
            }, onNoTapped = {})


          }
        }
      }
    }
  }

  private fun bindingAnimeUI() {
    imageView.setImageResource(imageResDrawable)
    titleTextView.text = title
    descriptionTextView.text = description
  }

  private fun initViewElements() {
    //find function is a short way
    // to use findViewById provide using ANKO
    imageView = findViewById<ImageView>(R.id.anime_image)
    titleTextView = findViewById<TextView>(R.id.title)
    descriptionTextView = findViewById<TextView>(R.id.description)
    loveItImageView = findViewById<ImageView>(R.id.love_it)
    hateItImageView = findViewById<ImageView>(R.id.hate_it)
  }

  private fun extractAnimeFrom(intent: Intent) {
    with(intent) {
      title = extras["TITLE_KEY"].toString()
      description = extras["DESCRIPTION_KEY"].toString()
      imdbLink = extras["IMDB_LINK_KEY"].toString()
      imageResDrawable = extras["IMAGE_KEY"] as Int
    }
  }


  override fun onOptionsItemSelected(item: MenuItem): Boolean {
    return when (item.itemId) {
      R.id.share -> {
        shareAnime()
        true
      }

      R.id.open_imdb -> {
        openAnimeInTheBrowser()
        true
      }
      else -> super.onOptionsItemSelected(item)
    }
  }

  override fun onCreateOptionsMenu(menu: Menu): Boolean {
    val inflater = menuInflater
    inflater.inflate(R.menu.anime_detail, menu)
    return true
  }

  private fun shareAnime() {
    share("$title \n $imdbLink", "Give a checkout to $title")
  }

  private fun openAnimeInTheBrowser(){
    browse(imdbLink)
  }

  private fun sendAnimeByEmail(emailAddress: String, subject: String, body: String) {
    email(emailAddress, subject, body)
  }

  private fun showShortToast(message:String){
    toast(message)
  }

  private fun showLongToast(message: String) {
    longToast(message)
  }

  private fun showAlert(messageResource: Int, onYesTapped: () -> Unit, onNoTapped: () -> Unit) {
    alert(messageResource) {
      //1. Creating the alert
      yesButton {
        //2. Handling yes button (This is optional)
        onYesTapped()
      }
      noButton {
        //3. Handling no button (This is optional)
        onNoTapped()
      }
    }.show() //4. Showing the alert
  }

  private fun showSelector(title: CharSequence, items: List<CharSequence>, onClick: (DialogInterface, Int) -> Unit) {
    selector(title,items,onClick)
  }
}
