/*
 * Copyright (c) 2018 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.kanime.activities

import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.Menu
import android.view.MenuItem
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import com.raywenderlich.kanime.R


class AnimeDetailActivity : AppCompatActivity() {

  private lateinit var title: String
  private lateinit var description: String
  private lateinit var imdbLink: String
  private var imageResDrawable: Int = 0

  private lateinit var imageView: ImageView
  private lateinit var titleTextView: TextView
  private lateinit var descriptionTextView: TextView
  private lateinit var loveItImageView: ImageView
  private lateinit var hateItImageView: ImageView

  companion object {
    //Love it constants
    private val INDEX_WATCH_IT_IN_DAY = 0
    private val INDEX_THE_BEST_OF_THE_BEST = 1
    private val INDEX_SEND_MAIL = 2

    //Hate it constants
    private val INDEX_BORED_TO_DEATH = 0
    private val INDEX_MY_GRANDMA = 1

  }

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    setContentView(R.layout.activity_anime_detail)

    initViewElements()

    extractAnimeFrom(intent)

    bindingAnimeUI()

    settingListeners()
  }

  private fun settingListeners() {
    loveItImageView.setOnClickListener {

      val options = listOf(getString(R.string.watched_id_in_a_day), getString(R.string.the_best), getString(R.string.tell_me_more_by_email))

      showSelector(getString(R.string.tell_me_more), options) { dialogInterface, selectedIndex ->
        when (selectedIndex) {
          INDEX_WATCH_IT_IN_DAY -> {
            //Easy and readable way to present a toast
            //Also, you can pass the String value instead of the string id resource
            showShortToast(getString(R.string.me_too))
          }
          INDEX_THE_BEST_OF_THE_BEST -> {
            // Did you remember Toast.makeText(context, message, Toast.LENGTH_LONG).show()?
            //  I don't
            showLongToast(getString(R.string.completely_agree))
          }
          INDEX_SEND_MAIL -> sendAnimeByEmail(getString(R.string.my_email), getString(R.string.template_email_body, title), getString(R.string.template_email_body, title))
        }
      }
    }

    hateItImageView.setOnClickListener {

      val options = listOf(getString(R.string.it_bored_to_death), getString(R.string.my_grandma_thinks_is_cool), getString(R.string.tell_me_more_by_email))

      //Creates a dialog with multiple options
      showSelector(getString(R.string.tell_me_more), options) { dialogInterface, selectedIndex ->

        when (selectedIndex) {
          INDEX_BORED_TO_DEATH -> showShortToast(getString(R.string.im_not_agree))
          INDEX_SEND_MAIL -> sendAnimeByEmail(getString(R.string.my_email), getString(R.string.template_negative_email_body, title), getString(R.string.template_negative_email_body, title))
          INDEX_MY_GRANDMA -> {

            showAlert(R.string.seriously, onYesTapped = {
              showShortToast(getString(R.string.smiley_face))
            }, onNoTapped = {})


          }
        }
      }
    }
  }

  private fun bindingAnimeUI() {
    imageView.setImageResource(imageResDrawable)
    titleTextView.text = title
    descriptionTextView.text = description
  }

  private fun initViewElements() {
    //find function is a short way
    // to use findViewById provide using ANKO
    imageView = findViewById<ImageView>(R.id.anime_image)
    titleTextView = findViewById<TextView>(R.id.title)
    descriptionTextView = findViewById<TextView>(R.id.description)
    loveItImageView = findViewById<ImageView>(R.id.love_it)
    hateItImageView = findViewById<ImageView>(R.id.hate_it)
  }

  private fun extractAnimeFrom(intent: Intent) {
    with(intent) {
      title = extras["TITLE_KEY"].toString()
      description = extras["DESCRIPTION_KEY"].toString()
      imdbLink = extras["IMDB_LINK_KEY"].toString()
      imageResDrawable = extras["IMAGE_KEY"] as Int
    }
  }


  override fun onOptionsItemSelected(item: MenuItem): Boolean {
    return when (item.itemId) {
      R.id.share -> {
        shareAnime()
        true
      }

      R.id.open_imdb -> {
        openAnimeInTheBrowser()
        true
      }
      else -> super.onOptionsItemSelected(item)
    }
  }

  override fun onCreateOptionsMenu(menu: Menu): Boolean {
    val inflater = menuInflater
    inflater.inflate(R.menu.anime_detail, menu)
    return true
  }

  private fun shareAnime() {
    val text = "$title \n $imdbLink"
    val subject = "Give a checkout to $title"
    //1. Creating the Intent
    val intent = Intent(android.content.Intent.ACTION_SEND)
    //2. Add extra data
    intent.type = "text/plain"
    intent.putExtra(android.content.Intent.EXTRA_SUBJECT, subject)
    intent.putExtra(android.content.Intent.EXTRA_TEXT, text)
    //3. open a list of possible apps that can handle this intent
    startActivity(Intent.createChooser(intent, null))
  }

  private fun openAnimeInTheBrowser() {
    val intent = Intent(Intent.ACTION_VIEW)
    intent.data = Uri.parse(imdbLink)
    startActivity(intent)
  }

  private fun sendAnimeByEmail(emailAddress: String, subject: String, body: String) {
    //1. Creating the Intent
    val intent = Intent(Intent.ACTION_SENDTO)
    //2. Add extra data
    intent.data = Uri.parse("mailto:")
    intent.putExtra(Intent.EXTRA_EMAIL, arrayOf(emailAddress))
    intent.putExtra(Intent.EXTRA_SUBJECT, subject)
    intent.putExtra(Intent.EXTRA_TEXT, body)
    //3. Open the email app
    startActivity(intent)
  }

  private fun showShortToast(message: String) {
    Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
  }

  private fun showLongToast(message: String) {
    Toast.makeText(this, message, Toast.LENGTH_LONG).show()
  }

  private fun showAlert(messageResource: Int, onYesTapped: () -> Unit, onNoTapped: () -> Unit) {
    val context = this
    //1. Creating the AlertDialog
    val alert = AlertDialog.Builder(context)
    //2. Setting the title
    alert.setTitle(messageResource)
    //3. Setting yes and no button handlers
    alert.setPositiveButton(android.R.string.yes) { dialog, which -> onYesTapped() }
    alert.setNegativeButton(android.R.string.no) { dialog, which -> onNoTapped() }
    //4. Building the alert and showing it
    alert.create().show()
  }

  private fun showSelector(title: CharSequence, items: List<CharSequence>, onClick: (DialogInterface, Int) -> Unit) {
    val context = this
    //1. Creating the AlertDialog
    val alertBuilder = AlertDialog.Builder(context)
    //2. Setting the title
    alertBuilder.setTitle(title)
    //3. Setting click handlers for each item of the list

    alertBuilder.setItems(Array(items.size) { itemIndex -> items[itemIndex].toString() }) { dialog, which ->
      onClick(dialog, which)
    }.show()
  }
}
